@@ -63,23 +63,24 @@ import org.slf4j.LoggerFactory;
  * Configuration:
  * 
  * <pre>
- * &lt;DataStore class="org.apache.jackrabbit.aws.ext.ds.CachingDataStore">
+ * &lt;DataStore class="org.apache.jackrabbit.aws.ext.ds.CachingDataStore"&gt;
  * 
- *     &lt;param name="{@link #setPath(String) path}" value="/data/datastore"/>
- *     &lt;param name="{@link #setConfig(String) config}" value="${rep.home}/backend.properties"/>
- *     &lt;param name="{@link #setCacheSize(long) cacheSize}" value="68719476736"/>
- *     &lt;param name="{@link #setSecret(String) secret}" value="123456"/>
- *     &lt;param name="{@link #setCachePurgeTrigFactor(double)}" value="0.95d"/>
- *     &lt;param name="{@link #setCachePurgeResizeFactor(double) cacheSize}" value="0.85d"/>
- *     &lt;param name="{@link #setMinRecordLength(int) minRecordLength}" value="1024"/>
- *     &lt;param name="{@link #setContinueOnAsyncUploadFailure(boolean) continueOnAsyncUploadFailure}" value="false"/>
- *     &lt;param name="{@link #setConcurrentUploadsThreads(int) concurrentUploadsThreads}" value="10"/>
- *     &lt;param name="{@link #setAsyncUploadLimit(int) asyncUploadLimit}" value="100"/>
- *     &lt;param name="{@link #setUploadRetries(int) uploadRetries}" value="3"/>
- *     &lt;param name="{@link #setTouchAsync(boolean) touchAsync}" value="false"/>
- *     &lt;param name="{@link #setProactiveCaching(boolean) proactiveCaching}" value="true"/>
- *     &lt;param name="{@link #setRecLengthCacheSize(int) recLengthCacheSize}" value="200"/>
- * &lt/DataStore>
+ *     &lt;param name="{@link #setPath(String) path}" value="/data/datastore"/&gt;
+ *     &lt;param name="{@link #setConfig(String) config}" value="${rep.home}/backend.properties"/&gt;
+ *     &lt;param name="{@link #setCacheSize(long) cacheSize}" value="68719476736"/&gt;
+ *     &lt;param name="{@link #setSecret(String) secret}" value="123456"/&gt;
+ *     &lt;param name="{@link #setCachePurgeTrigFactor(double)}" value="0.95d"/&gt;
+ *     &lt;param name="{@link #setCachePurgeResizeFactor(double) cacheSize}" value="0.85d"/&gt;
+ *     &lt;param name="{@link #setMinRecordLength(int) minRecordLength}" value="1024"/&gt;
+ *     &lt;param name="{@link #setContinueOnAsyncUploadFailure(boolean) continueOnAsyncUploadFailure}" value="false"/&gt;
+ *     &lt;param name="{@link #setConcurrentUploadsThreads(int) concurrentUploadsThreads}" value="10"/&gt;
+ *     &lt;param name="{@link #setAsyncUploadLimit(int) asyncUploadLimit}" value="100"/&gt;
+ *     &lt;param name="{@link #setUploadRetries(int) uploadRetries}" value="3"/&gt;
+ *     &lt;param name="{@link #setTouchAsync(boolean) touchAsync}" value="false"/&gt;
+ *     &lt;param name="{@link #setProactiveCaching(boolean) proactiveCaching}" value="true"/&gt;
+ *     &lt;param name="{@link #setRecLengthCacheSize(int) recLengthCacheSize}" value="200"/&gt;
+ * &lt;/DataStore&gt;
+ * </pre>
  */
 public abstract class CachingDataStore extends AbstractDataStore implements
         MultiDataStoreAware, AsyncUploadCallback, AsyncTouchCallback {
@@ -89,11 +90,6 @@ public abstract class CachingDataStore extends AbstractDataStore implements
      */
     private static final Logger LOG = LoggerFactory.getLogger(CachingDataStore.class);
 
-    /**
-     * The digest algorithm used to uniquely identify records.
-     */
-    private static final String DIGEST = "SHA-1";
-
     private static final String DS_STORE = ".DS_Store";
 
     /**
@@ -149,7 +145,7 @@ public abstract class CachingDataStore extends AbstractDataStore implements
     private File tmpDir;
 
     private String secret;
-    
+
     /**
      * Flag to indicate if lastModified is updated asynchronously.
      */
@@ -388,9 +384,9 @@ public abstract class CachingDataStore extends AbstractDataStore implements
 
     /**
      * Creates a new data record in {@link Backend}. The stream is first
-     * consumed and the contents are saved in a temporary file and the SHA-1
+     * consumed and the contents are saved in a temporary file and the {@link #DIGEST}
      * message digest of the stream is calculated. If a record with the same
-     * SHA-1 digest (and length) is found then it is returned. Otherwise new
+     * {@link #DIGEST} digest (and length) is found then it is returned. Otherwise new
      * record is created in {@link Backend} and the temporary file is moved in
      * place to {@link LocalCache}.
      * 
@@ -422,7 +418,7 @@ public abstract class CachingDataStore extends AbstractDataStore implements
             long currTime = System.currentTimeMillis();
             DataIdentifier identifier = new DataIdentifier(
                 encodeHexString(digest.digest()));
-            LOG.debug("SHA1 of [{}], length =[{}] took [{}]ms ",
+            LOG.debug("Digest of [{}], length =[{}] took [{}]ms ",
                 new Object[] { identifier, length, (currTime - startTime) });
             String fileName = getFileName(identifier);
             AsyncUploadCacheResult result = null;
@@ -469,7 +465,9 @@ public abstract class CachingDataStore extends AbstractDataStore implements
             if (getLength(identifier) > -1) {
                 LOG.trace("getRecord: [{}]  retrieved using getLength",
                     identifier);
-                touchInternal(identifier);
+                if (minModifiedDate > 0) {
+                    touchInternal(identifier);
+                }
                 usesIdentifier(identifier);
                 return new CachingDataRecord(this, identifier);
             } else if (asyncWriteCache.hasEntry(fileName, minModifiedDate > 0)) {
@@ -507,7 +505,9 @@ public abstract class CachingDataStore extends AbstractDataStore implements
                 LOG.trace(
                     "getRecordIfStored: [{}]  retrieved using recLenCache",
                     identifier);
-                touchInternal(identifier);
+                if (minModifiedDate > 0) {
+                    touchInternal(identifier);
+                }
                 usesIdentifier(identifier);
                 return new CachingDataRecord(this, identifier);
             } else {
@@ -517,7 +517,9 @@ public abstract class CachingDataStore extends AbstractDataStore implements
                         "getRecordIfStored :[{}]  retrieved from backend",
                         identifier);
                     recLenCache.put(identifier, length);
-                    touchInternal(identifier);
+                    if (minModifiedDate > 0) {
+                        touchInternal(identifier);
+                    }
                     usesIdentifier(identifier);
                     return new CachingDataRecord(this, identifier);
                 } catch (DataStoreException ignore) {
@@ -757,7 +759,7 @@ public abstract class CachingDataStore extends AbstractDataStore implements
                     if (retry <= uploadRetries) {
                         uploadRetryMap.put(identifier, retry);
                         LOG.info(
-                            "Retrying [{}] times failed upload for dataidentifer",
+                            "Retrying [{}] times failed upload for dataidentifer {}",
                             retry, identifier);
                         try {
                             backend.writeAsync(identifier, file, this);
@@ -765,7 +767,7 @@ public abstract class CachingDataStore extends AbstractDataStore implements
                             LOG.warn("exception", e);
                         }
                     } else {
-                        LOG.info("Retries [{}] exhausted for  dataidentifer.",
+                        LOG.info("Retries [{}] exhausted for  dataidentifer {}.",
                             (retry - 1), identifier);
                         uploadRetryMap.remove(identifier);
                     }
