@@ -32,31 +32,20 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 import java.util.Set;
-import java.util.concurrent.Executors;
-import java.util.concurrent.ThreadPoolExecutor;
 
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.IOUtils;
-import org.apache.jackrabbit.core.data.util.NamedThreadFactory;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-public class FSBackend implements Backend {
+public class FSBackend extends AbstractBackend {
 
     private Properties properties;
 
     private String fsPath;
 
-    private CachingDataStore store;
-
-    private String homeDir;
-
-    private String config;
-
     File fsPathDir;
 
-    private ThreadPoolExecutor asyncWriteExecuter;
-
     public static final String FS_BACKEND_PATH = "fsBackendPath";
 
     /**
@@ -72,10 +61,10 @@ public class FSBackend implements Backend {
     @Override
     public void init(CachingDataStore store, String homeDir, String config)
                     throws DataStoreException {
+        super.init(store, homeDir, config);
         Properties initProps = null;
         // Check is configuration is already provided. That takes precedence
         // over config provided via file based config
-        this.config = config;
         if (this.properties != null) {
             initProps = this.properties;
         } else {
@@ -98,12 +87,12 @@ public class FSBackend implements Backend {
 
     public void init(CachingDataStore store, String homeDir, Properties prop)
                     throws DataStoreException {
-        this.store = store;
-        this.homeDir = homeDir;
+        setDataStore(store);
+        setHomeDir(homeDir);
         this.fsPath = prop.getProperty(FS_BACKEND_PATH);
         if (this.fsPath == null || "".equals(this.fsPath)) {
             throw new DataStoreException("Could not initialize FSBackend from "
-                + config + ". [" + FS_BACKEND_PATH + "] property not found.");
+                + getConfig() + ". [" + FS_BACKEND_PATH + "] property not found.");
         }
         fsPathDir = new File(this.fsPath);
         if (fsPathDir.exists() && fsPathDir.isFile()) {
@@ -117,9 +106,6 @@ public class FSBackend implements Backend {
                     + fsPathDir.getAbsolutePath());
             }
         }
-        asyncWriteExecuter = (ThreadPoolExecutor) Executors.newFixedThreadPool(
-            10, new NamedThreadFactory("fs-write-worker"));
-
     }
 
     @Override
@@ -190,7 +176,7 @@ public class FSBackend implements Backend {
             throw new IllegalArgumentException(
                 "callback parameter cannot be null in asyncUpload");
         }
-        asyncWriteExecuter.execute(new Runnable() {
+        getAsyncWriteExecutor().execute(new Runnable() {
             @Override
             public void run() {
                 try {
@@ -268,7 +254,7 @@ public class FSBackend implements Backend {
             Thread.currentThread().setContextClassLoader(
                 getClass().getClassLoader());
 
-            asyncWriteExecuter.execute(new Runnable() {
+            getAsyncWriteExecutor().execute(new Runnable() {
                 @Override
                 public void run() {
                     try {
@@ -290,12 +276,6 @@ public class FSBackend implements Backend {
 
     }
 
-    @Override
-    public void close() throws DataStoreException {
-        asyncWriteExecuter.shutdownNow();
-
-    }
-
     @Override
     public Set<DataIdentifier> deleteAllOlderThan(long min)
                     throws DataStoreException {
@@ -457,8 +437,8 @@ public class FSBackend implements Backend {
                 }
                 if (lastModified < min) {
                     DataIdentifier id = new DataIdentifier(file.getName());
-                    if (store.confirmDelete(id)) {
-                        store.deleteFromCache(id);
+                    if (getDataStore().confirmDelete(id)) {
+                        getDataStore().deleteFromCache(id);
                         if (LOG.isInfoEnabled()) {
                             LOG.info("Deleting old file "
                                 + file.getAbsolutePath() + " modified: "
@@ -492,5 +472,4 @@ public class FSBackend implements Backend {
             }
         }
     }
-
 }
