@@ -88,6 +88,8 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Enumeration;
 import java.util.List;
 
 /**
@@ -107,7 +109,8 @@ abstract public class AbstractWebdavServlet extends HttpServlet implements DavCo
 
     /**
      * Name of the optional init parameter that defines the value of the
-     * 'WWW-Authenticate' header.<p/>
+     * 'WWW-Authenticate' header.
+     * <p>
      * If the parameter is omitted the default value
      * {@link #DEFAULT_AUTHENTICATE_HEADER "Basic Realm=Jackrabbit Webdav Server"}
      * is used.
@@ -282,6 +285,17 @@ abstract public class AbstractWebdavServlet extends HttpServlet implements DavCo
                 return;
             }
 
+            // JCR-4165: reject any content-coding in request until we can
+            // support it (see JCR-4166)
+            List<String> ces = getContentCodings(request);
+            if (!ces.isEmpty()) {
+                webdavResponse.setStatus(HttpServletResponse.SC_UNSUPPORTED_MEDIA_TYPE);
+                webdavResponse.setHeader("Accept-Encoding", "identity");
+                webdavResponse.setContentType("text/plain; charset=UTF-8");
+                webdavResponse.getWriter().println("Content-Encodings not supported, but received: " + ces);
+                webdavResponse.getWriter().flush();
+            }
+
             // check matching if=header for lock-token relevant operations
             DavResource resource = getResourceFactory().createResource(webdavRequest.getRequestLocator(), webdavRequest, webdavResponse);
             if (!isPreconditionValid(webdavRequest, resource)) {
@@ -845,7 +859,14 @@ abstract public class AbstractWebdavServlet extends HttpServlet implements DavCo
                     return DavServletResponse.SC_PRECONDITION_FAILED;
                 } else {
                     // overwrite existing resource
-                    destResource.getCollection().removeMember(destResource);
+                    DavResource col;
+                    try {
+                        col = destResource.getCollection();
+                    }
+                    catch (IllegalArgumentException ex) {
+                        return DavServletResponse.SC_BAD_GATEWAY;
+                    }
+                    col.removeMember(destResource);
                     status = DavServletResponse.SC_NO_CONTENT;
                 }
             } else {
@@ -1389,4 +1410,21 @@ abstract public class AbstractWebdavServlet extends HttpServlet implements DavCo
     protected OutputContext getOutputContext(DavServletResponse response, OutputStream out) {
         return new OutputContextImpl(response, out);
     }
+
+    private List<String> getContentCodings(HttpServletRequest request) {
+        List<String> result = Collections.emptyList();
+        for (@SuppressWarnings("unchecked")
+        Enumeration<String> ceh = request.getHeaders("Content-Encoding"); ceh.hasMoreElements();) {
+            for (String h : ceh.nextElement().split(",")) {
+                if (!h.trim().isEmpty()) {
+                    if (result.isEmpty()) {
+                        result = new ArrayList<String>();
+                    }
+                    result.add(h.trim());
+                }
+            }
+        }
+
+        return result;
+    }
 }
